using GLib;
using Gee;
using dbuster;

const string launcher_dir = "/usr/lib/dbuster";

server srv;
ArrayList<server.app> apps = null;
//ArrayList<string> apps_to_launch = null;

void main(){

	srv = new server();
	srv.init();
	
	apps = new ArrayList<server.app>();
	//apps_to_launch = new ArrayList<string>();
	
	//Launch all programs in launcher_dir
	var directory = File.new_for_path(launcher_dir);
    var enumerator = directory.enumerate_children (FILE_ATTRIBUTE_STANDARD_NAME, 0);
	
    FileInfo file_info;
    while ((file_info = enumerator.next_file ()) != null) {

		string name = file_info.get_name ();

		if (name == "dbuster-launch")//Ignore dbuster-launch
			continue;

		//Try to find binary
		string? ret = find_binary(name, launcher_dir);
		if (ret == null){
			log(@"Binary of '$name' not found!");
			continue;
		}
		
		launch_app(srv, name, ret);
    }
	srv.external_launch_request.connect(external_launch_request);//Launch instance when requested
	
	MainLoop loop = new MainLoop();
	loop.run();
}

void external_launch_request(string app_name, string[] args){
	foreach (server.app app in apps)
		if (app.name == app_name){
			app.launch(args);
			return;
		}
		
	//If requested app is not found in apps list, launch it now manually
	
	log (@"Got launch request for '$app_name', which I did not prepare. I will start it manually anyway...");
	
	string? bin = find_binary(app_name, launcher_dir);
	var unknown_app = new server.app(srv, app_name, bin);
	unknown_app.state_changed.connect((app)=>{
		if (app.state == enum_app_state.initialized)
			app.prepare();
		else if (app.state == enum_app_state.prepared){
			app.launch(args);
		}
	});
}

server.app launch_app(server srv, string name, string binary){
	var new_app = new server.app(srv, name, binary);
	apps.add(new_app);//Add to apps list
	new_app.state_changed.connect(()=>{app_state_changed(new_app);});
	return new_app;
}

static bool preparing = false;
void app_state_changed(server.app app){
	if (app.state == enum_app_state.initialized){
		if (!preparing){//If no other app is preparing right now, prepare immediately
			preparing = true;
			app.prepare();
		}
	}
	else if (app.state == enum_app_state.prepared){
		foreach (server.app a in apps)//Send prepare signal to an initialized, yet not prepared app
			if (a.state == enum_app_state.initialized){
				a.prepare();
				return;
			}
		preparing = false;//If there is no app ready for preparing, toggle 'preparing' so the next initialized app will be prepared directly
	}
	else if (app.state == enum_app_state.launched){//Launch next instance after current instance has been launched
		Timeout.add(1000, ()=>{
			
			apps.remove (launch_app(srv, app.name, app.binary)); // why remove the new app instance??? only works that way though -.-
			
			return false;
		});
	}
}

inline void log (string text){
	stdout.printf("dbuster-server: " + text + "\n");
}
